@@ -9,6 +9,7 @@ from mlflow.sagemaker import DEFAULT_IMAGE_NAME as IMAGE
 from mlflow.utils import cli_args
 from mlflow.utils.annotations import experimental
 import mlflow.models.docker_utils
+from mlflow.utils import env_manager as em
 
 
 @click.group("sagemaker")
@@ -478,6 +479,68 @@ def terminate_transform_job(job_name, region_name, archive, asynchronous, timeou
     )
 
 
+@commands.command("push-model")
+@click.option("--model-name", "-n", help="Sagemaker model name", required=True)
+@cli_args.MODEL_URI
+@click.option("--execution-role-arn", "-e", default=None, help="SageMaker execution role")
+@click.option("--bucket", "-b", default=None, help="S3 bucket to store model artifacts")
+@click.option("--image-url", "-i", default=None, help="ECR URL for the Docker image")
+@click.option(
+    "--region-name",
+    default="us-west-2",
+    help="Name of the AWS region in which to push the Sagemaker model",
+)
+@click.option(
+    "--vpc-config",
+    "-v",
+    help="Path to a file containing a JSON-formatted VPC configuration. This"
+    " configuration will be used when creating the new SageMaker model."
+    " For more information, see"
+    " https://docs.aws.amazon.com/sagemaker/latest/dg/API_VpcConfig.html",
+)
+@click.option(
+    "--flavor",
+    "-f",
+    default=None,
+    help=(
+        "The name of the flavor to use for deployment. Must be one of the following:"
+        " {supported_flavors}. If unspecified, a flavor will be automatically selected"
+        " from the model's available flavors.".format(
+            supported_flavors=mlflow.sagemaker.SUPPORTED_DEPLOYMENT_FLAVORS
+        )
+    ),
+)
+@experimental
+def push_model_to_sagemaker(
+    model_name,
+    model_uri,
+    execution_role_arn,
+    bucket,
+    image_url,
+    region_name,
+    vpc_config,
+    flavor,
+):
+    """
+    Push an MLflow model to Sagemaker model registry. Current active AWS account needs to have
+    correct permissions setup.
+    """
+    if vpc_config is not None:
+        with open(vpc_config, "r") as f:
+            vpc_config = json.load(f)
+
+    mlflow.sagemaker.push_model_to_sagemaker(
+        model_name=model_name,
+        model_uri=model_uri,
+        execution_role_arn=execution_role_arn,
+        bucket=bucket,
+        image_url=image_url,
+        region_name=region_name,
+        vpc_config=vpc_config,
+        flavor=flavor,
+    )
+
+
 @commands.command("run-local")
 @cli_args.MODEL_URI
 @click.option("--port", "-p", default=5000, help="Server port. [default: 5000]")
@@ -505,8 +568,9 @@ def run_local(model_uri, port, image, flavor):
 @click.option("--build/--no-build", default=True, help="Build the container if set.")
 @click.option("--push/--no-push", default=True, help="Push the container to AWS ECR if set.")
 @click.option("--container", "-c", default=IMAGE, help="image name")
+@cli_args.ENV_MANAGER
 @cli_args.MLFLOW_HOME
-def build_and_push_container(build, push, container, mlflow_home):
+def build_and_push_container(build, push, container, env_manager, mlflow_home):
     """
     Build new MLflow Sagemaker image, assign it a name, and push to ECR.
 
@@ -514,13 +578,16 @@ def build_and_push_container(build, push, container, mlflow_home):
     The image is built locally and it requires Docker to run.
     The image is pushed to ECR under current active AWS account and to current active AWS region.
     """
+    env_manager = env_manager or em.CONDA
     if not (build or push):
-        print("skipping both build and push, have nothing to do!")
+        click.echo("skipping both build and push, have nothing to do!")
     if build:
         sagemaker_image_entrypoint = """
         ENTRYPOINT ["python", "-c", "import sys; from mlflow.models import container as C; \
-        C._init(sys.argv[1])"]
-        """
+        C._init(sys.argv[1], '{env_manager}')"]
+        """.format(
+            env_manager=env_manager
+        )
 
         def setup_container(_):
             return "\n".join(
@@ -536,6 +603,7 @@ def build_and_push_container(build, push, container, mlflow_home):
             mlflow_home=os.path.abspath(mlflow_home) if mlflow_home else None,
             entrypoint=sagemaker_image_entrypoint,
             custom_setup_steps_hook=setup_container,
+            env_manager=env_manager,
         )
     if push:
         mlflow.sagemaker.push_image_to_ecr(container)
